@@ -6,7 +6,7 @@
 *
 * Created   :   14.02.2011
 *
-* Copyright 2007 - 2011 Zarafa Deutschland GmbH
+* Copyright 2007 - 2012 Zarafa Deutschland GmbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
@@ -59,6 +59,7 @@ class MAPIUtils {
      * @return array
      */
     public static function GetEmailRestriction($timestamp) {
+        // ATTENTION: ON CHANGING THIS RESTRICTION, MAPIUtils::IsInEmailSyncInterval() also needs to be changed
         $restriction = array ( RES_PROPERTY,
                           array (   RELOP => RELOP_GE,
                                     ULPROPTAG => PR_MESSAGE_DELIVERY_TIME,
@@ -74,6 +75,7 @@ class MAPIUtils {
      * Create a MAPI restriction to use in the calendar which will
      * return all future calendar items, plus those since $timestamp
      *
+     * @param MAPIStore  $store         the MAPI store
      * @param long       $timestamp     Timestamp since when to include messages
      *
      * @access public
@@ -88,6 +90,7 @@ class MAPIUtils {
         $props = MAPIMapping::GetAppointmentProperties();
         $props = getPropIdsFromStrings($store, $props);
 
+        // ATTENTION: ON CHANGING THIS RESTRICTION, MAPIUtils::IsInCalendarSyncInterval() also needs to be changed
         $restriction = Array(RES_OR,
              Array(
                    // OR
@@ -112,7 +115,7 @@ class MAPIUtils {
                    Array(RES_OR,
                          Array(
                                // OR
-                               // (EXIST(recurrence_enddate_property) && item[isRecurring] == true && item[end] >= start)
+                               // (EXIST(recurrence_enddate_property) && item[isRecurring] == true && recurrence_enddate_property >= start)
                                Array(RES_AND,
                                      Array(
                                            Array(RES_EXIST,
@@ -210,14 +213,90 @@ class MAPIUtils {
         );
     }
 
+    /**
+     * Checks if mapimessage is inside the synchronization interval
+     * also defined by MAPIUtils::GetEmailRestriction()
+     *
+     * @param MAPIStore       $store           mapi store
+     * @param MAPIMessage     $mapimessage     the mapi message to be checked
+     * @param long            $timestamp       the lower time limit
+     *
+     * @access public
+     * @return boolean
+     */
+    public static function IsInEmailSyncInterval($store, $mapimessage, $timestamp) {
+        $p = mapi_getprops($mapimessage, array(PR_MESSAGE_DELIVERY_TIME));
+
+        if (isset($p[PR_MESSAGE_DELIVERY_TIME]) && $p[PR_MESSAGE_DELIVERY_TIME] >= $timestamp) {
+            ZLog::Write(LOGLEVEL_DEBUG, "MAPIUtils->IsInEmailSyncInterval: Message is in the synchronization interval");
+            return true;
+        }
+
+        ZLog::Write(LOGLEVEL_WARN, "MAPIUtils->IsInEmailSyncInterval: Message is OUTSIDE the synchronization interval");
+        return false;
+    }
+
+    /**
+     * Checks if mapimessage is inside the synchronization interval
+     * also defined by MAPIUtils::GetCalendarRestriction()
+     *
+     * @param MAPIStore       $store           mapi store
+     * @param MAPIMessage     $mapimessage     the mapi message to be checked
+     * @param long            $timestamp       the lower time limit
+     *
+     * @access public
+     * @return boolean
+     */
+    public static function IsInCalendarSyncInterval($store, $mapimessage, $timestamp) {
+        // This is our viewing window
+        $start = $timestamp;
+        $end = 0x7fffffff; // infinite end
+
+        $props = MAPIMapping::GetAppointmentProperties();
+        $props = getPropIdsFromStrings($store, $props);
+
+        $p = mapi_getprops($mapimessage, array($props["starttime"], $props["endtime"], $props["recurrenceend"], $props["isrecurring"], $props["recurrenceend"]));
+
+        if (
+                (
+                    isset($p[$props["endtime"]]) && isset($p[$props["starttime"]]) &&
+
+                    //item.end > window.start && item.start < window.end
+                    $p[$props["endtime"]] > $start && $p[$props["starttime"]] < $end
+                )
+            ||
+                (
+                    isset($p[$props["isrecurring"]]) &&
+
+                    //(EXIST(recurrence_enddate_property) && item[isRecurring] == true && recurrence_enddate_property >= start)
+                    isset($p[$props["recurrenceend"]]) && $p[$props["isrecurring"]] == true && $p[$props["recurrenceend"]] >= $start
+                )
+            ||
+                (
+                    isset($p[$props["isrecurring"]]) && isset($p[$props["starttime"]]) &&
+
+                    //(!EXIST(recurrence_enddate_property) && item[isRecurring] == true && item[start] <= end)
+                    !isset($p[$props["recurrenceend"]]) && $p[$props["isrecurring"]] == true && $p[$props["starttime"]] <= $end
+                )
+           ) {
+            ZLog::Write(LOGLEVEL_DEBUG, "MAPIUtils->IsInCalendarSyncInterval: Message is in the synchronization interval");
+            return true;
+        }
+
+
+        ZLog::Write(LOGLEVEL_WARN, "MAPIUtils->IsInCalendarSyncInterval: Message is OUTSIDE the synchronization interval");
+        return false;
+    }
+
 
     /**
      * Handles recurring item for meeting request coming from tnef
      *
-     * @access public
-     *
      * @param array $mapiprops
      * @param array $props
+     *
+     * @access public
+     * @return
      */
     public static function handleRecurringItem(&$mapiprops, &$props) {
         $mapiprops[$props["isrecurringtag"]] = true;
@@ -237,14 +316,23 @@ class MAPIUtils {
     /**
      * Reads data of large properties from a stream
      *
-     * @access public
-     *
      * @param MAPIMessage $message
      * @param long $prop
+     *
+     * @access public
      * @return string
      */
     public static function readPropStream($message, $prop) {
         $stream = mapi_openproperty($message, $prop, IID_IStream, 0, 0);
+        $ret = mapi_last_hresult();
+        if ($ret == MAPI_E_NOT_FOUND) {
+            ZLog::Write(LOGLEVEL_DEBUG, sprintf("MAPIUtils->readPropStream: property 0x%s not found. It is either empty or not set. It will be ignored.", str_pad(dechex($prop), 8, 0, STR_PAD_LEFT)));
+            return "";
+        }
+        elseif ($ret) {
+            ZLog::Write(LOGLEVEL_ERROR, "MAPIUtils->readPropStream error opening stream: 0X%X", $ret);
+            return "";
+        }
         $data = "";
         $string = "";
         while(1) {
@@ -261,8 +349,10 @@ class MAPIUtils {
     /**
      * Checks if a store supports properties containing unicode characters
      *
-     * @access public
      * @param MAPIStore $store
+     *
+     * @access public
+     * @return
      */
     public static function IsUnicodeStore($store) {
         $supportmask = mapi_getprops($store, array(PR_STORE_SUPPORT_MASK));
