@@ -7,7 +7,7 @@
 *
 * Created   :   11.04.2011
 *
-* Copyright 2007 - 2011 Zarafa Deutschland GmbH
+* Copyright 2007 - 2012 Zarafa Deutschland GmbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
@@ -384,7 +384,7 @@ class ASDevice extends StateObject {
                 $ignored = $this->ignoredMessages;
                 $newMessages = array();
                 foreach ($ignored as $im) {
-                    if ($im->folderid = $folderid) {
+                    if ($im->folderid == $folderid) {
                         if ($id === false || $im->id === $id) {
                             $foundMessage = true;
                             if (count($this->ignoredMessageIds[$folderid]) == 1) {
